@@ -6,22 +6,7 @@ use strict;
 use Carp;
 use Regexp::RegGrp;
 
-our $VERSION            = '1.001_002';
-
-our @COMPRESS           = ( 'minify', 'pretty' );
-our $DEFAULT_COMPRESS   = 'pretty';
-
-our @BOOLEAN_ACCESSORS = (
-    'no_compress_comment',
-    'remove_copyright'
-);
-
-our @COPYRIGHT_ACCESSORS = (
-    'copyright',
-    'copyright_comment'
-);
-
-our $COPYRIGHT_COMMENT  = '\/\*((?>[^*]|\*[^/])*copyright(?>[^*]|\*[^/])*)\*\/';
+our $VERSION        = '1.000001';
 
 our $DICTIONARY     = {
     'STRING1'   => qr~"(?>(?:(?>[^"\\]+)|\\.|\\"|\\\s)*)"~,
@@ -40,87 +25,18 @@ our $MEDIA          = '\@media([^{}]+)\{((?:' . $IMPORT . '|' . $RULE . '|' . $W
 
 our $DECLARATION    = '((?>[^;:]+)):(?<=:)((?>[^;]*))(?:;|\s*$)';
 
-our $COMMENT        = '(\/\*[^*]*\*+([^/][^*]*\*+)*\/)';
+our $COMMENT        = '(\/\*[^*]*\*+([^\/][^*]*\*+)*\/)';
 
 our $PACKER_COMMENT = '\/\*\s*CSS::Packer\s*(\w+)\s*\*\/';
 
 our $CHARSET        = '^(\@charset)\s+(' . $DICTIONARY->{STRING1} . '|' . $DICTIONARY->{STRING2} . ');';
 
-our @REGGRPS        = ( 'whitespaces', 'url', 'import', 'declaration', 'rule', 'content_value', 'mediarules', 'global' );
-
 # --------------------------------------------------------------------------- #
 
-{
-    no strict 'refs';
-
-    foreach my $reggrp ( @REGGRPS ) {
-        next if defined *{ __PACKAGE__ . '::reggrp_' . $reggrp }{CODE};
-
-        *{ __PACKAGE__ . '::reggrp_' . $reggrp } = sub {
-            my ( $self ) = shift;
-
-            return $self->{ '_reggrp_' . $reggrp };
-        };
-    }
-
-    foreach my $field ( @BOOLEAN_ACCESSORS ) {
-        next if defined *{ __PACKAGE__ . '::' . $field }{CODE};
-
-        *{ __PACKAGE__ . '::' . $field} = sub {
-            my ( $self, $value ) = @_;
-
-            $self->{'_' . $field} = $value ? 1 : undef if ( defined( $value ) );
-
-            return $self->{'_' . $field};
-        };
-    }
-
-    foreach my $field ( @COPYRIGHT_ACCESSORS ) {
-        $field = '_' . $field if ( $field eq 'copyright_comment' );
-        next if defined *{ __PACKAGE__ . '::' . $field }{CODE};
-
-        *{ __PACKAGE__ . '::' . $field} = sub {
-            my ( $self, $value ) = @_;
-
-            if ( defined( $value ) and not ref( $value ) ) {
-                $value =~ s/^\s*|\s*$//gs;
-                $self->{'_' . $field} = $value;
-            }
-
-            my $ret = '';
-
-            if ( $self->{'_' . $field} ) {
-                $ret = '/* ' . $self->{'_' . $field} . ' */' . "\n";
-            }
-
-            return $ret;
-        };
-    }
-}
-
-sub compress {
-    my ( $self, $value ) = @_;
-
-    if ( defined( $value ) ) {
-        if ( grep( $value eq $_, @COMPRESS ) ) {
-            $self->{_compress} = $value;
-        }
-        elsif ( ! $value ) {
-            $self->{_compress} = undef;
-        }
-    }
-
-    $self->{_compress} ||= $DEFAULT_COMPRESS;
-
-    return $self->{_compress};
-}
-
 sub init {
-    my $class   = shift;
+    my $class    = shift;
     my $self    = {};
 
-    bless( $self, $class );
-
     $self->{content_value}->{reggrp_data} = [
         {
             regexp      => $DICTIONARY->{STRING1}
@@ -165,10 +81,15 @@ sub init {
                 my $submatches  = $_[0]->{submatches};
                 my $url         = $submatches->[0];
                 my $mediatype   = $submatches->[2];
+                my $opts        = $_[0]->{opts} || {};
 
-                my $compress    = $self->compress();
+                my $compress    = _get_opt( $opts, 'compress' );
 
-                $self->reggrp_url()->exec( \$url );
+                # I don't like this, but
+                # $self->{url}->{reggrp}->exec( \$url );
+                # will not work. It isn't initialized jet.
+                # If someone has a better idea, please let me know
+                $self->_process_wrapper( 'url', \$url, $opts );
 
                 $mediatype =~ s/^\s*|\s*$//gs;
                 $mediatype =~ s/\s*,\s*/,/gsm;
@@ -185,14 +106,19 @@ sub init {
                 my $submatches  = $_[0]->{submatches};
                 my $key         = $submatches->[0];
                 my $value       = $submatches->[1];
+                my $opts        = $_[0]->{opts} || {};
 
-                my $compress    = $self->compress();
+                my $compress    = _get_opt( $opts, 'compress' );
 
                 $key    =~ s/^\s*|\s*$//gs;
                 $value  =~ s/^\s*|\s*$//gs;
 
                 if ( $key eq 'content' ) {
-                    $self->reggrp_content_value->exec( \$value );
+                    # I don't like this, but
+                    # $self->{content_value}->{reggrp}->exec( \$value );
+                    # will not work. It isn't initialized jet.
+                    # If someone has a better idea, please let me know
+                    $self->_process_wrapper( 'content_value', \$value, $opts );
                 }
                 else {
                     $value =~ s/\s*,\s*/,/gsm;
@@ -213,8 +139,9 @@ sub init {
                 my $submatches  = $_[0]->{submatches};
                 my $selector    = $submatches->[0];
                 my $declaration = $submatches->[1];
+                my $opts        = $_[0]->{opts} || {};
 
-                my $compress    = $self->compress();
+                my $compress    = _get_opt( $opts, 'compress' );
 
                 $selector =~ s/^\s*|\s*$//gs;
                 $selector =~ s/\s*,\s*/,/gsm;
@@ -222,7 +149,11 @@ sub init {
 
                 $declaration =~ s/^\s*|\s*$//gs;
 
-                $self->reggrp_declaration()->exec( \$declaration );
+                # I don't like this, but
+                # $self->{declaration}->{reggrp}->exec( \$declaration );
+                # will not work. It isn't initialized jet.
+                # If someone has a better idea, please let me know
+                $self->_process_wrapper( 'declaration', \$declaration, $opts );
 
                 my $store = $selector . '{' . ( $compress eq 'pretty' ? "\n" : '' ) . $declaration . '}' .
                     ( $compress eq 'pretty' ? "\n" : '' );
@@ -245,10 +176,9 @@ sub init {
             regexp      => $CHARSET,
             replacement => sub {
                 my $submatches  = $_[0]->{submatches};
+                my $opts        = $_[0]->{opts} || {};
 
-                my $compress    = $self->compress();
-
-                return $submatches->[0] . " " . $submatches->[1] . ( $compress eq 'pretty' ? "\n" : '' );
+                return $submatches->[0] . " " . $submatches->[1] . ( $opts->{compress} eq 'pretty' ? "\n" : '' );
             }
         },
         {
@@ -257,13 +187,18 @@ sub init {
                 my $submatches  = $_[0]->{submatches};
                 my $mediatype   = $submatches->[0];
                 my $mediarules  = $submatches->[1];
+                my $opts        = $_[0]->{opts} || {};
 
-                my $compress    = $self->compress();
+                my $compress    = _get_opt( $opts, 'compress' );
 
                 $mediatype =~ s/^\s*|\s*$//gs;
                 $mediatype =~ s/\s*,\s*/,/gsm;
 
-                $self->reggrp_mediarules()->exec( \$mediarules );
+                # I don't like this, but
+                # $self->{mediarules}->{reggrp}->exec( \$mediarules );
+                # will not work. It isn't initialized jet.
+                # If someone has a better idea, please let me know
+                $self->_process_wrapper( 'mediarules', \$mediarules, $opts );
 
                 return '@media ' . $mediatype . '{' . ( $compress eq 'pretty' ? "\n" : '' ) .
                     $mediarules . '}' . ( $compress eq 'pretty' ? "\n" : '' );
@@ -274,12 +209,14 @@ sub init {
 
 
     map {
-        $self->{ '_reggrp_' . $_ } = Regexp::RegGrp->new(
+        $self->{$_}->{reggrp} = Regexp::RegGrp->new(
             {
                 reggrp => $self->{$_}->{reggrp_data}
             }
         );
-    } @REGGRPS;
+    } ( 'whitespaces', 'url', 'import', 'declaration', 'rule', 'content_value', 'mediarules', 'global' );
+
+    bless( $self, $class );
 
     return $self;
 }
@@ -319,44 +256,56 @@ sub minify {
         $css = ref( $input ) ? $input : \$input;
     }
 
-    if ( ref( $opts ) eq 'HASH' ) {
-        foreach my $field ( @BOOLEAN_ACCESSORS ) {
-            $self->$field( $opts->{$field} ) if ( defined( $opts->{$field} ) );
-        }
-
-        foreach my $field ( 'compress', 'copyright' ) {
-            $self->$field( $opts->{$field} ) if ( defined( $opts->{$field} ) );
-        }
+    if ( ref( $opts ) ne 'HASH' ) {
+        carp( 'Second argument must be a hashref of options! Using defaults!' ) if ( $opts );
+        $opts = { compress => 'pretty', no_compress_comment => 0 };
+    }
+    else {
+        $opts->{compress}               = grep( $opts->{compress}, ( 'minify', 'pretty' ) ) ? $opts->{compress} : 'pretty';
+        $opts->{no_compress_comment}    = $opts->{no_compress_comment} ? 1 : 0;
     }
 
-    my $copyright_comment = '';
-
-    if ( ${$css} =~ /$COPYRIGHT_COMMENT/ism ) {
-        $copyright_comment = $1;
-    }
-    # Resets copyright_comment() if there is no copyright comment
-    $self->_copyright_comment( $copyright_comment );
-
-    if ( not $self->no_compress_comment() and ${$css} =~ /$PACKER_COMMENT/ ) {
+    if ( not $opts->{no_compress_comment} and ${$css} =~ /$PACKER_COMMENT/ ) {
         my $compress = $1;
         if ( $compress eq '_no_compress_' ) {
             return ( $cont eq 'scalar' ) ? ${$css} : undef;
         }
 
-        $self->compress( $compress );
+        $opts->{compress} = grep( $compress, ( 'minify', 'pretty' ) ) ? $compress : $opts->{compress};
     }
 
     ${$css} =~ s/$COMMENT/ /gsm;
 
-    $self->reggrp_global()->exec( $css );
-
-    if ( not $self->remove_copyright() ) {
-        ${$css} = ( $self->copyright() || $self->_copyright_comment() ) . ${$css};
-    }
+    $self->{global}->{reggrp}->exec( $css, $opts );
 
     return ${$css} if ( $cont eq 'scalar' );
 }
 
+sub _process_wrapper {
+    my ( $self, $reg_name, $in, $opts ) = @_;
+
+    $self->{$reg_name}->{reggrp}->exec( $in, $opts );
+}
+
+sub _restore_wrapper {
+    my ( $self, $reg_name, $in ) = @_;
+
+    $self->{$reg_name}->{reggrp}->restore_stored( $in );
+}
+
+sub _get_opt {
+    my ( $opts_hash, $opt ) = @_;
+
+    $opts_hash  ||= {};
+    $opt       ||= '';
+
+    my $ret = '';
+
+    $ret = $opts_hash->{$opt} if ( defined( $opts_hash->{$opt} ) );
+
+    return $ret;
+}
+
 1;
 
 __END__
@@ -367,7 +316,7 @@ CSS::Packer - Another CSS minifier
 
 =head1 VERSION
 
-Version 1.001_002
+Version 1.000001
 
 =head1 DESCRIPTION
 
@@ -390,7 +339,7 @@ For backward compatibility it is still possible to call 'minify' as a function:
     CSS::Packer::minify( $scalarref, $opts );
 
 First argument must be a scalarref of CSS-Code.
-Second argument must be a hashref of options. Possible options are:
+Second argument must be a hashref of options. The only option is
 
 =over 4
 
@@ -421,26 +370,6 @@ to
 
     a{color:black;}div{width:100px;}
 
-=item copyright
-
-You can add a copyright notice at the top of the script.
-
-=item remove_copyright
-
-If there is a copyright notice in a comment it will only be removed if this
-option is set to a true value. Otherwise the first comment that contains the
-word "copyright" will be added at the top of the packed script. A copyright
-comment will be overwritten by a copyright notice defined with the copyright
-option.
-
-=item no_compress_comment
-
-If not set to a true value it is allowed to set a CSS comment that
-prevents the input being packed or defines a compression level.
-
-    /* CSS::Packer _no_compress_ */
-    /* CSS::Packer pretty */
-
 =back
 
 =head1 AUTHOR
@@ -449,8 +378,8 @@ Merten Falk, C<< <nevesenin at cpan.org> >>
 
 =head1 BUGS
 
-Please report any bugs or feature requests through
-the web interface at L<http://github.com/nevesenin/css-packer-perl/issues>.  I will be notified, and then you'll
+Please report any bugs or feature requests to C<bug-css-packer at rt.cpan.org>, or through
+the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=CSS-Packer>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
 =head1 SUPPORT
